{******************************************************************
  PlnCtaUDF 2.0 - UDF para Plano de Conta em InterBase/FireBird

  Autor....: Daniel Pereira Guimares
  E-mail...: tecnobyte@ulbrajp.com.br
  Home-Page: www.ulbrajp.com.br/~tecnobyte

  Colaboradores:
    Paulo Henrique Albanez

  Esta biblioteca  Open-Source!

  Voc pode usar, alterar e copiar esta biblioteca para onde e quando
  quizer. Sua distribuio  livre, sendo exigido apenas que os
  crditos do autor sejam mantidos.

  Gostaria de receber suas crticas e sugestes. Envie seus comentrios
  para o e-mail: tecnobyte@ulbrajp.com.br
********************************************************************}

library PlnCtaUDF;

uses
  SysUtils;

type
  TLevels = array[1..10] of integer;

function StrOfChr(Ch: Char; Count: integer): string;
var
  I: integer;
begin
  Result := '';
  for I := 1 to Count do
    Result := Result + Ch;
end;

function InternalPadZero(Value: LongInt; const Count: byte): ShortString;
var
  I, Len: byte;
begin
  if Value < 0 then
    Value := Value * (-1);
    
  Result := IntToStr(Value);
  
  Len := Length(Result);
  
  if Count < Len then
    Result := StrOfChr('*', Count)
  else
    for I := Len + 1 to Count do
      Result := '0' + Result;
end;

{ A procedure InternalClear limpa a Cta, eliminando caracteres estranhos }
function InternalClear(Cta: PChar): PChar; stdcall;
var
  Ch: PChar;
begin
  Result := Cta;

  Ch := Cta;

  { Posiciona no primeiro digito numerico, pois uma conta
    precisa iniciar-se com numero }
  while (Ch^ <> #0) and (not (Ch^ in ['0'..'9'])) do
    Inc(Ch);

  { Copia os caracteres validos }
  while Ch^ <> #0 do begin
    if Ch^ in ['.', '0'..'9'] then begin
      Cta^ := Ch^;
      Inc(Cta);
    end;
    Inc(Ch);
  end;

  Cta^ := #0;
end;

function InternalDecode(S: PChar): TLevels; stdcall;
var
  Level: byte;
  ProximoCh: PChar;
  Temp: string;
begin
  FillChar(Result, SizeOf(Result), 0);

  { Posiciona no primeiro digito }
  while (S^ <> #0) and (not (S^ in ['0'..'9'])) do
    Inc(S);

  { Pega todos os niveis }
  Level := Low(TLevels);
  Temp := '';
  while (S^ <> #0) and (Level <= High(TLevels)) do begin

    if S^ in ['0'..'9'] then
      Temp := Temp + S^;

    ProximoCh := S + 1; { Pega o proximo caractere }
    if ((S^ = '.') or (ProximoCh^ = #0)) and (Temp <> '') then begin
      Result[Level] := StrToInt(Temp);
      Inc(Level);
      Temp := '';
    end;

    Inc(S);
  end;
end;

function InternalFormat(const Levels, MaxValues: TLevels): string;
 stdcall;
var
  I: byte;
begin
  Result := '';

  { Verifica se em algum nivel a conta apresenta valor maior que
    o permitido. Se houver, cancela. }
  for I := Low(TLevels) to High(TLevels) do
    if Levels[I] > MaxValues[I] then
      Exit;

  { Um nivel onde o valor maximo e zero indica o fim da formatacao }
  I := Low(TLevels);
  while (I <= High(TLevels)) and (Levels[I] > 0) do begin
    if Result <> '' then
      Result := Result + '.';
    Result := Result + InternalPadZero(Levels[I], Length(IntToStr(MaxValues[I])));
    Inc(I);
  end;
end;

function PlnCtaFormat(Cta, Fmt: PChar): PChar; cdecl; export;
var
  Levels, MaxValues: TLevels;
  S: ShortString;
begin
  Result := Cta; { Usa Cta para retorno }

  Levels := InternalDecode(Cta);
  MaxValues := InternalDecode(Fmt);

  S := InternalFormat(Levels, MaxValues);

  if S <> '' then
    StrPCopy(Result, S)
  else
    Result := nil;
end;

function PlnCtaMain(Cta: PChar): PChar; cdecl; export;
var
  Ch: PChar;
begin
  Result := InternalClear(Cta);

  Ch := Cta + StrLen(Result) - 1; { Ultimo caractere }

  { Posiciona no ultimo ponto que estiver antes do ultimo digito }

  { Procura o ultimo digito }
  while (Ch > Result) and (not (Ch^ in ['0'..'9'])) do
    Dec(Ch);

  { Procura o ultimo ponto }
  while (Ch > Result) and (Ch^ <> '.') do
    Dec(Ch);

  { Marca a posicao de Ch como fim da string }
  Ch^ := #0;

  { Retorna nil se a conta nao possui pai, ou seja, se for
    uma conta de nivel 1 }
  if Ch = Result then
    Result := nil;
end;

function PlnCtaRoot(Cta: PChar): PChar; cdecl; export;
var
  Ch: PChar;
begin
  Result := InternalClear(Cta);

  Ch := Result + StrLen(Result) - 1; { Ultimo caractere }

  { Ignora tudo que existir apos o ultimo digito }
  while (Ch > Result) and (not (Ch^ in ['0'..'9'])) do begin
    Ch^ := #0;
    Dec(Ch);
  end;

  { Procura o primeiro ponto }
  Ch := Result;
  while (Ch^ <> '.') and (Ch^ <> #0) do
    Inc(Ch);

  { Se encontrou um ponto... }
  if Ch^ = '.' then
    Ch^ := #0
  else
    Result := nil;
end;

exports
  PlnCtaFormat,
  PlnCtaMain,
  PlnCtaRoot;

end.
